/** RtpDataQueue.h
===========================================================================
MODULE				: RtpDataQueue

DESCRIPTIONS		: Data queue that synchronises adding data and removing data between different threads
			

REVISION HISTORY	: Version 1.0


COPYRIGHT			: (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
the property of CSIR, Meraka Institute and has been 
classified as PUBLIC.
===========================================================================
*/

#pragma once

// STL
#include <queue>

// RTVC
#include <Shared/MediaSample.h>

class RtpDataQueue
{
public:

	RtpDataQueue()
		:m_bHasBeenSyncedUsingRtcp(false)
	{;}

	~RtpDataQueue()
	{
		CAutoLock lock(&m_lock); 
		while ( !m_qSamples.empty() )
		{
			MediaSample* pSample = m_qSamples.front();
			m_qSamples.pop();
			delete pSample;
		}
	}

	void addMediaSample(unsigned char* data, unsigned dataSize, double dStartTime, bool bHasBeenRtcpSynchronised)
	{
		CAutoLock lock(&m_lock); 
		if (!m_bHasBeenSyncedUsingRtcp)
		{
			if (bHasBeenRtcpSynchronised)
			{
				m_bHasBeenSyncedUsingRtcp = true;
				// Create a marked sample
				MediaSample* pMediaSample = MediaSample::createMediaSample(data, dataSize, dStartTime, true);
				m_qSamples.push(pMediaSample);
			}
		}
		// Else add a normal sample to the queue
		MediaSample* pMediaSample = MediaSample::createMediaSample(data, dataSize, dStartTime /*, false*/);
		m_qSamples.push(pMediaSample);
	}

	MediaSample* getNextSample()
	{
		CAutoLock lock(&m_lock); 
		if (!m_qSamples.empty())
		{
			MediaSample* pSample = m_qSamples.front();
			m_qSamples.pop();
			return pSample;
		}
		return NULL;
	}

	bool hasSamples()
	{
		CAutoLock lock(&m_lock); 
		return !m_qSamples.empty();
	}

private:
	bool m_bHasBeenSyncedUsingRtcp;

	CCritSec m_lock;
	std::queue<MediaSample*> m_qSamples;
};
