#ifndef __RTVC_RTSP_CLIENT_SESSION
#define __RTVC_RTSP_CLIENT_SESSION

// STL
#include <string>
#include <vector>

#include <liveMedia.hh>

// Forward decl
class RtspSourceFilter;

// LiveMedia
class UsageEnvironment;
class RTSPClient;
class MediaSession;
class LiveDirectShowSink;

class RtspClientSession
{
public:
	RtspClientSession(RtspSourceFilter* pSourceFilter, bool bStreamUsingTcp = true);
	virtual ~RtspClientSession(){}

	/// Accessor
	std::string LastError() const { return m_sLastError; }

	bool start(std::string sUrl);
	
	int TimeoutS() const { return m_nTimeoutS; }
	void TimeoutS(int val) { m_nTimeoutS = val; }

	bool initialiseRtspSession( const char* szUrl );
	bool setupAndPlayRtspSession();

	void shutdown();

	static void subsessionByeHandler(void* clientData) 
	{
		RtspClientSession* pClientSession = (RtspClientSession*) clientData;
		pClientSession->shutdown();
		// TODO: still need to notify DS filter that a stop has occurred!!!!!
	}

	void beginQOSMeasurement();

	void scheduleNextQOSMeasurement();
	void periodicQOSMeasurement();
	static void doPeriodicQOSMeasurement(void* clientData) ;

	//static void subsessionAfterPlaying(void* clientData) 
	//{
	//	// Begin by closing this media subsession's stream:
	//	MediaSubsession* subsession = (MediaSubsession*)clientData;
	//	Medium::close(subsession->sink);
	//	subsession->sink = NULL;

	//	// Next, check whether *all* subsessions' streams have now been closed:
	//	MediaSession& session = subsession->parentSession();
	//	MediaSubsessionIterator iter(session);
	//	while ((subsession = iter.next()) != NULL) {
	//		if (subsession->sink != NULL) return; // this subsession is still active
	//	}

	//	// All subsessions' streams have now been closed
	//	//sessionAfterPlaying();
	//}
protected:

	class qosMeasurementRecord {
	public:
		qosMeasurementRecord(struct timeval const& startTime, RTPSource* src)
			: fSource(src), fNext(NULL),
			kbits_per_second_min(1e20), kbits_per_second_max(0),
			kBytesTotal(0.0),
			packet_loss_fraction_min(1.0), packet_loss_fraction_max(0.0),
			totNumPacketsReceived(0), totNumPacketsExpected(0) {
				measurementEndTime = measurementStartTime = startTime;

				RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
				// Assume that there's only one SSRC source (usually the case):
				RTPReceptionStats* stats = statsIter.next(True);
				if (stats != NULL) {
					kBytesTotal = stats->totNumKBytesReceived();
					totNumPacketsReceived = stats->totNumPacketsReceived();
					totNumPacketsExpected = stats->totNumPacketsExpected();
				}
		}
		virtual ~qosMeasurementRecord() { delete fNext; }

		void periodicQOSMeasurement(struct timeval const& timeNow);

	public:
		RTPSource* fSource;
		qosMeasurementRecord* fNext;

	public:
		struct timeval measurementStartTime, measurementEndTime;
		double kbits_per_second_min, kbits_per_second_max;
		double kBytesTotal;
		double packet_loss_fraction_min, packet_loss_fraction_max;
		unsigned totNumPacketsReceived, totNumPacketsExpected;
	};
protected:
	// Override this if you want to do anything after session creation
	virtual bool afterSessionInit()
	{
		return true;
	}

	// This should return false for stream init where the media type does not need to be detected
	virtual bool needToPlayStream()
	{
		return true;
	}

	virtual bool isMediaTypeKnown()
	{
		return true;
	}
	
	

	bool createRtpSources();
	bool setupStreams();
	bool createRtpReceivers();
	bool startPlayingStreams();


	void closeMediaSinks();

	qosMeasurementRecord* qosRecordHead;
	unsigned qosMeasurementIntervalMS;
	TaskToken qosMeasurementTimerTask;

	// LiveMedia 
	UsageEnvironment* env;
	RTSPClient* m_pRtspClient;
	MediaSession* m_pSession;
	unsigned m_statusCode;
	unsigned nextQOSMeasurementUSecs;
	int m_nTimeoutS;
	std::vector<LiveDirectShowSink*> m_vLiveMediaSinks;
	// DirectShow stuff
	RtspSourceFilter* m_pSourceFilter;
	bool m_bStreamUsingTCP;
	// URL
	std::string m_sUrl;
	std::string m_sLastError;
};
#endif