/** RtspDataSession.h
===========================================================================
MODULE          : RtspDataSession: 

DESCRIPTIONS    : LiveMedia data session
				  Receives RTP packets and forwards these to the DirectShow pipeline

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

// STL
#include <map>
#include <string>
#include <vector>

// LiveMedia
#include <liveMedia.hh>

// Forward
class RtpPacketManager;

class RtspDataSession
{
public:
	/// Constructor
	RtspDataSession(RtpPacketManager* pRtpPacketManager);

	/// This method sets up the RTSP/RTP connections and starts the liveMedia event loop that retrieves data
	bool startRetrievingData(const std::string sUrl, int nTimeOut = -1);

	/// Call this method to end the life media event loop
	void setWatchVariable()
	{
		m_watchVariable = 1;
	}

	std::string getLastError()
	{
		return m_sLastError;
	}

	/// Shutdown RTSP session cleanly
	void shutdown();

	/// Bye Handler
	static void subsessionByeHandler(void* clientData) 
	{
		RtspDataSession* pSnifferSession = (RtspDataSession*) clientData;
		pSnifferSession->shutdown();
	}

private:

	bool playStreams(UsageEnvironment* pEnv, MediaSession* pSession);

	bool createReceivers(MediaSession* pSession);
	/// This method creates a mapping between the liveMedia media subsession which contains all the media information
	/// And our rtvc::MediaSubsession, which stores all data required to create the output pins
	/// We could just have used the liveMedia class and created our output pins using that but this method seperates the livemedia 
	/// Code from our own and makes our Outputpin implementation independent of the liveMedia library
	bool mapLiveMediaToRtvc(MediaSession* pSession);

	/// Currently only H263 streams need to be played back so that we can extract the width and the height from the raw data stream
	bool needToPlayStreams(MediaSession* pSubsession);
	
	// Packet Manager
	RtpPacketManager* m_pRtpPacketManager;

	/// Rtsp client
	RTSPClient* m_pRtspClient;

	/// MediaSession
	MediaSession* m_pSession;

	/// TCP
	bool m_bStreamUsingTCP;

	/// LiveMediaEventLoop watch variable
	char m_watchVariable;

	// Stores last error
	std::string m_sLastError;

	// Stores media subsessions
	std::vector<MediaSubsession*> m_vMediaSubsessions;
};
