/** RtspSnifferSession.cpp
===========================================================================
MODULE          : RtspSnifferSession

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#include "RtspSnifferSession.h"

// LiveMedia
#include "BasicUsageEnvironment.hh"

// RTVC
#include "RtspHelper.h"
#include "RtvcRtpSink.h"

RtspSnifferSession::RtspSnifferSession()
	:m_pRtspClient(NULL),
	m_pSession(NULL),
	m_bStreamUsingTCP(true),
	m_watchVariable(0),
	m_sLastError("")
{;}

RtspSnifferSession::~RtspSnifferSession()
{
	if (m_pRtspClient && m_pSession)
	// Cleanup RTSP Media session
	RtspHelper::shutdownMediaSession(m_pRtspClient, m_pSession);
}

bool RtspSnifferSession::getMediaSessionInformation(const std::string sUrl, int nTimeOut)
{
	const char* szUrl = sUrl.c_str();

	// Setup environment
	TaskScheduler* pScheduler = BasicTaskScheduler::createNew();
	UsageEnvironment* pEnv = BasicUsageEnvironment::createNew(*pScheduler);

	bool bSuccess = true;

	do
	{
		// Create client
		m_pRtspClient = RTSPClient::createNew(*pEnv);

		if (!RtspHelper::createMediaSession(sUrl, m_pRtspClient, &m_pSession, m_sLastError))
		{
			bSuccess = false;
			break;
		}
		
		// Iterate over session and add subsessions to vectors
		MediaSubsession* pSubsession = NULL;
		MediaSubsessionIterator it(*m_pSession);
		while ((pSubsession = it.next()) != NULL)
		{
			m_vSubsessions.push_back(pSubsession);
		}
		// Create our mappings
		/*if (!mapLiveMediaToRtvc(m_pSession))
		{
			bSuccess = false;
			break;
		}*/

		//bSuccess = playStreamsIfNecessary(pEnv, m_pSession);
		break;
	}while (true);
	return bSuccess;
}

void RtspSnifferSession::shutdown()
{
	// Cleanup RTSP Media session
	RtspHelper::shutdownMediaSession(m_pRtspClient, m_pSession);
}

//bool RtspSnifferSession::playStreamsIfNecessary(UsageEnvironment* pEnv, MediaSession* pSession)
//{
//	if (needToPlayStreams(pSession))
//	{
//		// Setup and play the streams 
//		if (!RtspHelper::createRtpSources(pEnv, pSession))
//		{
//			m_sLastError = "Failed to create RTP sources";
//			return false;
//		}
//
//		// Setup RTP source
//		if (!RtspHelper::setupStreams(m_pRtspClient, pSession, m_bStreamUsingTCP))
//		{
//			m_sLastError = "Failed to setup RTP streams";
//			return false;
//		}
//
//		if (!createReceivers(pSession))
//		{
//			m_sLastError =  "Failed to create RTP receivers: " +  std::string(pEnv->getResultMsg());
//			return false;
//		}
//
//		// Play streams
//		if (!m_pRtspClient->playMediaSession(*pSession)) 
//		{
//			m_sLastError =  "Failed to start playing session: " +  std::string(pEnv->getResultMsg());
//			return false;
//		}
//
//		// Start the liveMedia eventloop
//		pEnv->taskScheduler().doEventLoop(&m_watchVariable); // does not return	until watch variable is set
//	}
//	return true;
//}

//bool RtspSnifferSession::createReceivers(MediaSession* pSession)
//{
//	// Create and start media sinks for each subsession:
//	bool madeProgress = False;
//	MediaSubsessionIterator iter(*m_pSession);
//	MediaSubsession *pSubsession;
//	
//	while ((pSubsession = iter.next()) != NULL) 
//	{
//		// Create our RTPSinks that will receive the data
//		if (pSubsession->readSource() == NULL) continue;// was not initiated
//
//		if ( strcmp(pSubsession->mediumName(), "video") == 0 && 
//			 ((strcmp(pSubsession->codecName(), "H263-1998") == 0)||
//			  (strcmp(pSubsession->codecName(), "H263-2000") == 0))
//			)
//		{
//			int nSinkBufferSize = 20000;
//			// Create our H263Sniffer
//			m_pFormatSniffer = new H263FormatSniffer(&(m_subsessionMap[pSubsession]), m_watchVariable);
//			RtvcRtpSink<H263FormatSniffer>* pSink = new RtvcRtpSink<H263FormatSniffer>(
//					pSession->envir(), 
//					0 /*Not interested in source id in sniffing session*/, 
//					nSinkBufferSize, 
//					m_pFormatSniffer
//					);
//			pSubsession->sink = pSink;
//		}
//		else
//		{
//			// Don't create any sinks since there's no data to sniff
//			continue;
//		}
//
//		// TODO Set after playing pointers
//		pSubsession->sink->startPlaying(*(pSubsession->readSource()), NULL/*subsessionAfterPlaying*/, NULL/*subsession*/);
//
//		// Also set a handler to be called if a RTCP "BYE" arrives
//		// for this subsession:
//		if (pSubsession->rtcpInstance() != NULL) 
//		{
//			pSubsession->rtcpInstance()->setByeHandler(&RtspSnifferSession::subsessionByeHandler, this/*subsession*/);
//		}
//		madeProgress = True;
//	}
//	return madeProgress;
//}

//bool RtspSnifferSession::mapLiveMediaToRtvc(MediaSession* pSession)
//{
//	// Check if all subsession could get mapped
//	return ((nLiveMediaSubsessionCount == m_subsessionMap.size())? true : false);
//}


bool RtspSnifferSession::needToPlayStreams(MediaSession* pSession)
{
	MediaSubsessionIterator iter(*pSession);
	MediaSubsession *pSubsession = NULL;
	bool bFailed = false;
	while ((pSubsession = iter.next()) != NULL && !bFailed) 
	{
		const char* szMedium = pSubsession->mediumName();
		const char* szCodec = pSubsession->codecName();
		if ((strcmp(szMedium, "video")==0) && 
			((strcmp(szCodec, "H263-1998")==0)||(strcmp(szCodec, "H263-2000")==0))
			)
		{
			return true;
		}
	}
	return false;
}
