/** @file
===========================================================================
MODULE				: MultiIOBaseFilter

DESCRIPTIONS		: This base class takes care of doing most of the menial work when writing a transform filter that has multiple input and output pins.

REVISION HISTORY	: Version 1.0

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/

#pragma once

#include <streams.h>

class CMultiIOOutputPin : public CBaseOutputPin
{
	friend class CMultiIOBaseFilter;
	friend class CMultiIOInputPin;
public:
	CMultiIOOutputPin(CMultiIOBaseFilter* pFilter, CCritSec* pLock, HRESULT* phr, LPCWSTR pName, int nIndex);
	virtual ~CMultiIOOutputPin(void);

	// lifetime management for pins is normally delegated to the filter, but
	// we need to be able to create and delete them independently, so keep 
	// a separate refcount.
	STDMETHODIMP_(ULONG) NonDelegatingRelease()
	{
		return CUnknown::NonDelegatingRelease();
	}
	STDMETHODIMP_(ULONG) NonDelegatingAddRef()
	{
		return CUnknown::NonDelegatingAddRef();
	}

	HRESULT CompleteConnect(IPin *pReceivePin);

	// From CBaseOutputPin
	virtual HRESULT DecideBufferSize(IMemAllocator* pAlloc, ALLOCATOR_PROPERTIES* pRequestProperties);
	// From CBasePin
	virtual HRESULT CheckMediaType(const CMediaType* pMediaType);

	HRESULT GetMediaType(int iPosition, CMediaType* pmt);

	// inherited from IQualityControl via CBasePin
	STDMETHODIMP Notify(IBaseFilter * pSender, Quality q);

	IMemAllocator* GetAllocator(){return m_pAllocator;}
	IMemInputPin* GetInputPin(){return m_pInputPin;}

	// Overridden to create output queue's
	HRESULT Active();
	HRESULT Inactive();
	HRESULT Deliver(IMediaSample *pMediaSample);
	HRESULT DeliverEndOfStream();
	HRESULT DeliverBeginFlush();
	HRESULT DeliverEndFlush();
	HRESULT DeliverNewSegment(
		REFERENCE_TIME tStart,
		REFERENCE_TIME tStop,
		double dRate);

protected:
	CMultiIOBaseFilter* m_pFilter;
	int m_nIndex;

	COutputQueue* m_pOutputQueue;
};
