/** @file

MODULE				: PicScalerBase

TAG					: PSB

FILE NAME			: PicScalerBase.h

DESCRIPTION			: This class is the base class defining the minimum interface	
						and properties for all derived implementations to scale the
						dimensions of an image either up or down. The instantiation 
						process must set the parameters and limits to the images such 
						that the Scale()implementation only requires source image 
						data pointers. All memory new/delete must be handled outside 
						of these objects. The scale method is pure virtual and must be 
						implemented by derived classes.

REVISION HISTORY	:

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/
#ifndef _PICSCALERBASE_H
#define _PICSCALERBASE_H

#pragma once

/*
===========================================================================
  Class definition.
===========================================================================
*/
class PicScalerBase
{
public:
	// Construction and destruction.
	PicScalerBase(void) {_widthOut = 1; _heightOut = 1; _widthIn = 1; _heightIn = 1; }
	PicScalerBase(int widthOut, int heightOut, int widthIn, int heightIn) 
	{	_widthOut = widthOut; _heightOut = heightOut; _widthIn = widthIn; _heightIn = heightIn; }
	virtual ~PicScalerBase(void) {}

	// Interface.
	virtual int Scale(void* pOutImg, void* pInImg) = 0;

	// Member interface.
	int	GetOutWidth(void)		{ return(_widthOut); }
	int	GetOutHeight(void)	{ return(_heightOut); }
	int	GetInWidth(void)		{ return(_widthIn); }
	int	GetInHeight(void)		{ return(_heightIn); }
  
	void SetOutDimensions(int widthOut, int heightOut)	{_widthOut = widthOut; _heightOut = heightOut; }
	void SetInDimensions(int widthIn, int heightIn)			{_widthIn = widthIn; _heightIn = heightIn; }

// Private methods.
protected:
	// Members.
	int	_widthOut;	/// Image dimensions to fit to.
	int	_heightOut;
	int _widthIn;		/// Image dimensions to scale from.
	int _heightIn;

};//end PicScalerBase.

#endif	// _PICSCALERBASE_H
