/** @file

MODULE				: RGBtoYUV420Converter

TAG					: RGBYUVC

FILE NAME			: RGBtoYUV420Converter.h

DESCRIPTION			: Colour convertions to YUV420 are required on the input of
					  all video codecs. For embedded applications only some
					  combinations of colour depths are required. This class is 
					  the base class defining the minimum interface	and 
					  properties for all derived classes. The conversion method
					  is pure virtual and must be implemented by derived classes.

REVISION HISTORY	: 

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/
#ifndef _RGBTOYUV420CONVERTER_H
#define _RGBTOYUV420CONVERTER_H

#pragma once

#ifdef _BUILD_FOR_SHORT
typedef short yuvType;
#else
typedef signed char yuvType ;
#endif

/*
===========================================================================
  Class definition.
===========================================================================
*/
class RGBtoYUV420Converter
{
public:
	// Construction and destruction.
	RGBtoYUV420Converter(void) {_width = 0; _height = 0; }
	RGBtoYUV420Converter(int width, int height) {_width = width; _height = height;}
	virtual ~RGBtoYUV420Converter(void) {}

	// Interface.
	virtual void Convert(void* pRgb, void* pY, void* pU, void* pV) = 0;

	// Member interface.
	int	GetWidth(void)		{ return(_width); }
	int	GetHeight(void)		{ return(_height); }

	void	SetDimensions(int width, int height)	{_width = width; _height = height;}

protected:
	// Members.
	int	_width;
	int	_height;

};//end RGBtoYUV420Converter.

#endif	// _RGBTOYUV420CONVERTER_H
